package cn.iocoder.yudao.module.member.job;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.member.dal.dataobject.level.MemberUserLevelConfigDO;
import cn.iocoder.yudao.module.member.dal.dataobject.score.MemberUserScoreDO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.redis.scoreRule.ScoreRuleRedisDao;
import cn.iocoder.yudao.module.member.enums.MemberLevelStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.service.level.MemberUserLevelConfigService;
import cn.iocoder.yudao.module.member.service.score.MemberUserScoreService;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 会员等级详情定时任务
 */
@Component
@Slf4j
public class userLevelDetailsTask implements JobHandler {
    /**
     * 每天凌晨00:00AM和12:00AM 分别执行一次
     * @param param 参数
     * @return
     * @throws Exception
     */

    @Resource
    private MemberUserLevelConfigService memberUserLevelConfigService;
    @Resource
    private MemberUserService userService;
    @Resource
    private MemberUserScoreLogService memberUserScoreLogService;
    @Override
    public String execute(String param) throws Exception {
        log.info("level config expire task running");
        LambdaQuery<MemberUserLevelConfigDO> userLevelConfigDoLambdaQuery = new LambdaQuery<>();
        userLevelConfigDoLambdaQuery.eq(MemberUserLevelConfigDO::getStatus, MemberLevelStatusEnum.ENABLED.getValue());
        List<MemberUserLevelConfigDO> todoList = memberUserLevelConfigService.selectList(userLevelConfigDoLambdaQuery);
        log.info("level config expire task, to expire rule count :{}", todoList.size());
        if (CollectionUtils.isEmpty(todoList)) {
            return "success";
        } else {
            todoList.forEach(userLevelConfigDO -> {
                LambdaQuery<MemberUserScoreLogDO> memberUserScoreLogDoLambdaQuery =  new LambdaQuery<>();
                //获取满足条件的所有用户
                if (userLevelConfigDO.getStartDate()!=null){
                    memberUserScoreLogDoLambdaQuery.ge(MemberUserScoreLogDO::getCreateTime, userLevelConfigDO.getStartDate());
                }else{
                    memberUserScoreLogDoLambdaQuery.ge(MemberUserScoreLogDO::getCreateTime, DateUtils.getNextNDayStart(new Date(), -(userLevelConfigDO.getAccumulationPeriod())));
                }
            });
        }
        return "success";
    }
}
